@@ -18,63 +18,63 @@
 //! ```rust
 //! extern crate time;
 //! extern crate sqlite3;
-//!
+//! 
 //! use time::Timespec;
-//!
-//!
+//! 
+//! 
 //! use sqlite3::{
 //!     DatabaseConnection,
-//!     DatabaseUpdate,
 //!     Query,
 //!     ResultRowAccess,
 //!     SqliteResult,
+//!     StatementUpdate,
 //! };
-//!
-//! #[deriving(Show)]
+//! 
+//! #[derive(Debug)]
 //! struct Person {
 //!     id: i32,
 //!     name: String,
 //!     time_created: Timespec,
 //!     // TODO: data: Option<Vec<u8>>
 //! }
-//!
+//! 
 //! pub fn main() {
 //!     match io() {
-//!         Ok(ppl) => println!("Found people: {}", ppl),
+//!         Ok(ppl) => println!("Found people: {:?}", ppl),
 //!         Err(oops) => panic!(oops)
 //!     }
 //! }
-//!
+//! 
 //! fn io() -> SqliteResult<Vec<Person>> {
 //!     let mut conn = try!(DatabaseConnection::in_memory());
-//!
+//! 
 //!     try!(conn.exec("CREATE TABLE person (
 //!                  id              SERIAL PRIMARY KEY,
 //!                  name            VARCHAR NOT NULL,
 //!                  time_created    TIMESTAMP NOT NULL
 //!                )"));
-//!
+//! 
 //!     let me = Person {
 //!         id: 0,
-//!         name: "Dan".to_string(),
+//!         name: format!("Dan"),
 //!         time_created: time::get_time(),
 //!     };
 //!     {
 //!         let mut tx = try!(conn.prepare("INSERT INTO person (name, time_created)
 //!                            VALUES ($1, $2)"));
-//!         let changes = try!(conn.update(&mut tx, &[&me.name, &me.time_created]));
+//!         let changes = try!(tx.update(&[&me.name, &me.time_created]));
 //!         assert_eq!(changes, 1);
 //!     }
-//!
+//! 
 //!     let mut stmt = try!(conn.prepare("SELECT id, name, time_created FROM person"));
-//!
+//! 
 //!     let mut ppl = vec!();
 //!     try!(stmt.query(
-//!         &[], |row| {
+//!         &[], &mut |row| {
 //!             ppl.push(Person {
 //!                 id: row.get("id"),
 //!                 name: row.get("name"),
-//!                 time_created: row.get(2u)
+//!                 time_created: row.get(2)
 //!             });
 //!             Ok(())
 //!         }));
@@ -84,18 +84,24 @@
 
 #![crate_name = "sqlite3"]
 #![crate_type = "lib"]
-#![feature(unsafe_destructor)]
 #![warn(missing_docs)]
 
 extern crate libc;
 extern crate time;
 
-use std::error::{Error, FromError};
-use std::fmt::Show;
-use std::io::{IoError, OtherIoError};
+#[macro_use]
+extern crate bitflags;
+
+#[macro_use]
+extern crate enum_primitive;
+
+use std::error::{Error};
+use std::fmt::Display;
+use std::fmt;
 
 pub use core::Access;
 pub use core::{DatabaseConnection, PreparedStatement, ResultSet, ResultRow};
+pub use core::{ColIx, ParamIx};
 pub use types::{FromSql, ToSql};
 
 use self::SqliteErrorCode::SQLITE_MISUSE;
@@ -113,15 +119,14 @@ pub mod ffi;
 pub mod access;
 
 /// Mix in `update()` convenience function.
-pub trait DatabaseUpdate {
+pub trait StatementUpdate {
     /// Execute a statement after binding any parameters.
-    fn update<'db, 's>(&'db mut self,
-                       stmt: &'s mut PreparedStatement<'s>,
-                       values: &[&ToSql]) -> SqliteResult<uint>;
+    fn update(&mut self,
+              values: &[&ToSql]) -> SqliteResult<u64>;
 }
 
 
-impl DatabaseUpdate for core::DatabaseConnection {
+impl StatementUpdate for core::PreparedStatement {
     /// Execute a statement after binding any parameters.
     ///
     /// When the statement is done, The [number of rows
@@ -132,20 +137,18 @@ impl DatabaseUpdate for core::DatabaseConnection {
     /// `UPDATE`).
     ///
     /// [changes]: http://www.sqlite.org/c3ref/changes.html
-    fn update<'db, 's>(&'db mut self,
-                       stmt: &'s mut PreparedStatement<'s>,
-                       values: &[&ToSql]) -> SqliteResult<uint> {
+    fn update(&mut self,
+              values: &[&ToSql]) -> SqliteResult<u64> {
         let check = {
-            try!(bind_values(stmt, values));
-            let mut results = stmt.execute();
-            match results.step() {
+            try!(bind_values(self, values));
+            let mut results = self.execute();
+            match try!(results.step()) {
                 None => Ok(()),
-                Some(Ok(_row)) => Err(SqliteError {
+                Some(_row) => Err(SqliteError {
                     kind: SQLITE_MISUSE,
                     desc: "unexpected SQLITE_ROW from update",
                     detail: None
-                }),
-                Some(Err(e)) => Err(e)
+                })
             }
         };
         check.map(|_ok| self.changes())
@@ -154,39 +157,44 @@ impl DatabaseUpdate for core::DatabaseConnection {
 
 
 /// Mix in `query()` convenience function.
-pub trait Query<'s> {
+pub trait Query<F>
+    where F: FnMut(&mut ResultRow) -> SqliteResult<()>
+{
     /// Process rows from a query after binding parameters.
-    fn query(&'s mut self,
+    fn query(&mut self,
              values: &[&ToSql],
-             each_row: |&mut ResultRow|: 's -> SqliteResult<()>
+             each_row: &mut F
              ) -> SqliteResult<()>;
 }
 
-impl<'s> Query<'s> for core::PreparedStatement<'s> {
+impl<F> Query<F> for core::PreparedStatement
+    where F: FnMut(&mut ResultRow) -> SqliteResult<()>
+{
     /// Process rows from a query after binding parameters.
     ///
     /// For call `each_row(row)` for each resulting step,
     /// exiting on `Err`.
-    fn query(&'s mut self,
+    fn query(&mut self,
              values: &[&ToSql],
-             each_row: |&mut ResultRow|: 's -> SqliteResult<()>
-             ) -> SqliteResult<()> {
+             each_row: &mut F
+             ) -> SqliteResult<()>
+    {
         try!(bind_values(self, values));
         let mut results = self.execute();
         loop {
-            match results.step() {
+            match try!(results.step()) {
                 None => break,
-                Some(Ok(ref mut row)) => try!(each_row(row)),
-                Some(Err(e)) => return Err(e)
+                Some(ref mut row) => try!(each_row(row)),
             }
         }
         Ok(())
     }
 }
 
-fn bind_values<'db>(s: &'db mut PreparedStatement, values: &[&ToSql]) -> SqliteResult<()> {
+fn bind_values(s: &mut PreparedStatement, values: &[&ToSql]) -> SqliteResult<()> {
     for (ix, v) in values.iter().enumerate() {
-        try!(v.to_sql(s, ix + 1));
+        let p = ix as ParamIx + 1;
+        try!(v.to_sql(s, p));
     }
     Ok(())
 }
@@ -199,21 +207,21 @@ pub trait ResultRowAccess {
     /// # Panic
     ///
     /// Panics if there is no such column or value.
-    fn get<I: RowIndex + Show + Clone, T: FromSql>(&mut self, idx: I) -> T;
+    fn get<I: RowIndex + Display + Clone, T: FromSql>(&mut self, idx: I) -> T;
 
     /// Try to get `T` type result value from `idx`th column of a row.
-    fn get_opt<I: RowIndex + Show + Clone, T: FromSql>(&mut self, idx: I) -> SqliteResult<T>;
+    fn get_opt<I: RowIndex + Display + Clone, T: FromSql>(&mut self, idx: I) -> SqliteResult<T>;
 }
 
-impl<'s, 'r> ResultRowAccess for core::ResultRow<'s, 'r> {
-    fn get<I: RowIndex + Show + Clone, T: FromSql>(&mut self, idx: I) -> T {
+impl<'res, 'row> ResultRowAccess for core::ResultRow<'res, 'row> {
+    fn get<I: RowIndex + Display + Clone, T: FromSql>(&mut self, idx: I) -> T {
         match self.get_opt(idx.clone()) {
             Ok(ok) => ok,
             Err(err) => panic!("retrieving column {}: {}", idx, err)
         }
     }
 
-    fn get_opt<I: RowIndex + Show + Clone, T: FromSql>(&mut self, idx: I) -> SqliteResult<T> {
+    fn get_opt<I: RowIndex + Display + Clone, T: FromSql>(&mut self, idx: I) -> SqliteResult<T> {
         match idx.idx(self) {
             Some(idx) => FromSql::from_sql(self, idx),
             None => Err(SqliteError {
@@ -232,12 +240,12 @@ impl<'s, 'r> ResultRowAccess for core::ResultRow<'s, 'r> {
 /// [RowIndex]: http://www.rust-ci.org/sfackler/rust-postgres/doc/postgres/trait.RowIndex.html
 pub trait RowIndex {
     /// Try to convert `self` to an index into a row.
-    fn idx(&self, row: &mut ResultRow) -> Option<uint>;
+    fn idx(&self, row: &mut ResultRow) -> Option<ColIx>;
 }
 
-impl RowIndex for uint {
+impl RowIndex for ColIx {
     /// Index into a row directly by uint.
-    fn idx(&self, _row: &mut ResultRow) -> Option<uint> { Some(*self) }
+    fn idx(&self, _row: &mut ResultRow) -> Option<ColIx> { Some(*self) }
 }
 
 impl RowIndex for &'static str {
@@ -245,8 +253,8 @@ impl RowIndex for &'static str {
     ///
     /// *TODO: figure out how to use lifetime of row rather than
     /// `static`.*
-    fn idx(&self, row: &mut ResultRow) -> Option<uint> {
-        let mut ixs = range(0, row.column_count());
+    fn idx(&self, row: &mut ResultRow) -> Option<ColIx> {
+        let mut ixs = 0 .. row.column_count();
         ixs.find(|ix| row.with_column_name(*ix, false, |name| name == *self))
     }
 }
@@ -266,40 +274,42 @@ pub type SqliteResult<T> = Result<T, SqliteError>;
 /// `Some(...)` or `None` from `ResultSet::next()`.
 ///
 /// [codes]: http://www.sqlite.org/c3ref/c_abort.html
-#[deriving(Show, PartialEq, Eq, FromPrimitive, Copy)]
-#[allow(non_camel_case_types)]
-#[allow(missing_docs)]
-pub enum SqliteErrorCode {
-    SQLITE_ERROR     =  1,
-    SQLITE_INTERNAL  =  2,
-    SQLITE_PERM      =  3,
-    SQLITE_ABORT     =  4,
-    SQLITE_BUSY      =  5,
-    SQLITE_LOCKED    =  6,
-    SQLITE_NOMEM     =  7,
-    SQLITE_READONLY  =  8,
-    SQLITE_INTERRUPT =  9,
-    SQLITE_IOERR     = 10,
-    SQLITE_CORRUPT   = 11,
-    SQLITE_NOTFOUND  = 12,
-    SQLITE_FULL      = 13,
-    SQLITE_CANTOPEN  = 14,
-    SQLITE_PROTOCOL  = 15,
-    SQLITE_EMPTY     = 16,
-    SQLITE_SCHEMA    = 17,
-    SQLITE_TOOBIG    = 18,
-    SQLITE_CONSTRAINT= 19,
-    SQLITE_MISMATCH  = 20,
-    SQLITE_MISUSE    = 21,
-    SQLITE_NOLFS     = 22,
-    SQLITE_AUTH      = 23,
-    SQLITE_FORMAT    = 24,
-    SQLITE_RANGE     = 25,
-    SQLITE_NOTADB    = 26
+enum_from_primitive! {
+    #[derive(Debug, PartialEq, Eq, Copy, Clone)]
+    #[allow(non_camel_case_types)]
+    #[allow(missing_docs)]
+    pub enum SqliteErrorCode {
+        SQLITE_ERROR     =  1,
+        SQLITE_INTERNAL  =  2,
+        SQLITE_PERM      =  3,
+        SQLITE_ABORT     =  4,
+        SQLITE_BUSY      =  5,
+        SQLITE_LOCKED    =  6,
+        SQLITE_NOMEM     =  7,
+        SQLITE_READONLY  =  8,
+        SQLITE_INTERRUPT =  9,
+        SQLITE_IOERR     = 10,
+        SQLITE_CORRUPT   = 11,
+        SQLITE_NOTFOUND  = 12,
+        SQLITE_FULL      = 13,
+        SQLITE_CANTOPEN  = 14,
+        SQLITE_PROTOCOL  = 15,
+        SQLITE_EMPTY     = 16,
+        SQLITE_SCHEMA    = 17,
+        SQLITE_TOOBIG    = 18,
+        SQLITE_CONSTRAINT= 19,
+        SQLITE_MISMATCH  = 20,
+        SQLITE_MISUSE    = 21,
+        SQLITE_NOLFS     = 22,
+        SQLITE_AUTH      = 23,
+        SQLITE_FORMAT    = 24,
+        SQLITE_RANGE     = 25,
+        SQLITE_NOTADB    = 26
+    }
 }
 
 /// Error results
-#[deriving(Show, PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq)]
 pub struct SqliteError {
     /// kind of error, by code
     pub kind: SqliteErrorCode,
@@ -309,36 +319,39 @@ pub struct SqliteError {
     pub detail: Option<String>
 }
 
-impl Error for SqliteError {
-    fn description(&self) -> &str { self.desc }
-    fn detail(&self) -> Option<String> { self.detail.clone() }
-    fn cause(&self) -> Option<&Error> { None }
-}
-
-impl FromError<SqliteError> for IoError {
-    /// TODO: specialize the kind according to sqlite errors
-    fn from_error(sqlerr: SqliteError) -> IoError {
-        IoError {
-            kind: OtherIoError,
-            desc: sqlerr.desc,
-            detail: sqlerr.detail
+impl Display for SqliteError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self.detail {
+            Some(ref x) => f.write_fmt(format_args!("{} ({})", x, self.kind as u32)),
+            None => f.write_fmt(format_args!("{} ({})", self.desc, self.kind as u32))
         }
     }
 }
 
-
-/// Fundamental Datatypes
-#[deriving(Show, PartialEq, Eq, FromPrimitive, Copy)]
-#[allow(non_camel_case_types)]
-#[allow(missing_docs)]
-pub enum ColumnType {
-    SQLITE_INTEGER = 1,
-    SQLITE_FLOAT   = 2,
-    SQLITE_TEXT    = 3,
-    SQLITE_BLOB    = 4,
-    SQLITE_NULL    = 5
+impl SqliteError {
+    /// Get a detailed description of the error
+    pub fn detail(&self) -> Option<String> { self.detail.clone() }
 }
 
+impl Error for SqliteError {
+    fn description(&self) -> &str { self.desc }
+    fn cause(&self) -> Option<&Error> { None }
+}
+
+
+/// Fundamental Datatypes
+enum_from_primitive! {
+    #[derive(Debug, PartialEq, Eq, Copy, Clone)]
+    #[allow(non_camel_case_types)]
+    #[allow(missing_docs)]
+    pub enum ColumnType {
+        SQLITE_INTEGER = 1,
+        SQLITE_FLOAT   = 2,
+        SQLITE_TEXT    = 3,
+        SQLITE_BLOB    = 4,
+        SQLITE_NULL    = 5
+    }
+}
 
 #[cfg(test)]
 mod bind_tests {
@@ -365,15 +378,15 @@ mod bind_tests {
                 try!(tx.bind_text(2, "Jane Doe"));
                 try!(tx.bind_text(3, "345 e Walnut"));
                 let mut results = tx.execute();
-                assert!(results.step().is_none());
+                assert!(results.step().ok().unwrap().is_none());
             }
             assert_eq!(database.changes(), 1);
 
             let mut q = try!(database.prepare("select * from test order by id"));
             let mut rows = q.execute();
             match rows.step() {
-                Some(Ok(ref mut row)) => {
-                    assert_eq!(row.get::<uint, i32>(0), 1);
+                Ok(Some(ref mut row)) => {
+                    assert_eq!(row.get::<u32, i32>(0), 1);
                     // TODO let name = q.get_text(1);
                     // assert_eq!(name.as_slice(), "John Doe");
                 },
@@ -381,8 +394,8 @@ mod bind_tests {
             }
 
             match rows.step() {
-                Some(Ok(ref mut row)) => {
-                    assert_eq!(row.get::<uint, i32>(0), 2);
+                Ok(Some(ref mut row)) => {
+                    assert_eq!(row.get::<u32, i32>(0), 2);
                     //TODO let addr = q.get_text(2);
                     // assert_eq!(addr.as_slice(), "345 e Walnut");
                 },
@@ -392,31 +405,34 @@ mod bind_tests {
         }
         match go() {
             Ok(_) => (),
-            Err(e) => panic!("oops! {}", e)
+            Err(e) => panic!("oops! {:?}", e)
         }
     }
 
-    fn with_query<T>(sql: &str, f: |rows: &mut ResultSet| -> T) -> SqliteResult<T> {
-        let mut db = try!(DatabaseConnection::in_memory());
+    fn with_query<T, F>(sql: &str, mut f: F) -> SqliteResult<T>
+        where F: FnMut(&mut ResultSet) -> T
+    {
+        let db = try!(DatabaseConnection::in_memory());
         let mut s = try!(db.prepare(sql));
         let mut rows = s.execute();
-        Ok(f(&mut rows))
+        let x = f(&mut rows);
+        return Ok(x);
     }
 
     #[test]
     fn named_rowindex() {
-        fn go() -> SqliteResult<(uint, i32)> {
+        fn go() -> SqliteResult<(u32, i32)> {
             let mut count = 0;
-            let mut sum = 0;
+            let mut sum = 0i32;
 
             with_query("select 1 as col1
                        union all
                        select 2", |rows| {
                 loop {
                     match rows.step() {
-                        Some(Ok(ref mut row)) => {
+                        Ok(Some(ref mut row)) => {
                             count += 1;
-                            sum += row.get("col1")
+                            sum += row.column_int(0);
                         },
                         _ => break
                     }
@@ -433,17 +449,17 @@ mod bind_tests {
             let mut conn = try!(DatabaseConnection::in_memory());
             conn.exec("CREATE gobbledygook")
         };
-        
+
         let go = || match io() {
             Ok(_) => panic!(),
             Err(oops) => {
-                format!("{}: {}: {}",
+                format!("{:?}: {}: {}",
                         oops.kind, oops.desc,
                         oops.detail.unwrap())
             }
         };
 
         let expected = "SQLITE_ERROR: sqlite3_exec: near \"gobbledygook\": syntax error";
-        assert_eq!(go().as_slice(), expected)
+        assert_eq!(go(), expected.to_string())
     }
 }
