@@ -6,13 +6,13 @@ use time::Timespec;
 
 use sqlite3::{
     DatabaseConnection,
-    DatabaseUpdate,
     Query,
     ResultRowAccess,
     SqliteResult,
+    StatementUpdate,
 };
 
-#[deriving(Show)]
+#[derive(Debug)]
 struct Person {
     id: i32,
     name: String,
@@ -22,7 +22,7 @@ struct Person {
 
 pub fn main() {
     match io() {
-        Ok(ppl) => println!("Found people: {}", ppl),
+        Ok(ppl) => println!("Found people: {:?}", ppl),
         Err(oops) => panic!(oops)
     }
 }
@@ -38,25 +38,25 @@ fn io() -> SqliteResult<Vec<Person>> {
 
     let me = Person {
         id: 0,
-        name: "Dan".to_string(),
+        name: format!("Dan"),
         time_created: time::get_time(),
     };
     {
         let mut tx = try!(conn.prepare("INSERT INTO person (name, time_created)
                            VALUES ($1, $2)"));
-        let changes = try!(conn.update(&mut tx, &[&me.name, &me.time_created]));
+        let changes = try!(tx.update(&[&me.name, &me.time_created]));
         assert_eq!(changes, 1);
-    }
+    };
 
     let mut stmt = try!(conn.prepare("SELECT id, name, time_created FROM person"));
 
     let mut ppl = vec!();
     try!(stmt.query(
-        &[], |row| {
+        &[], &mut |row| {
             ppl.push(Person {
                 id: row.get("id"),
                 name: row.get("name"),
-                time_created: row.get(2u)
+                time_created: row.get(2)
             });
             Ok(())
         }));
